@@ -69,7 +69,9 @@ class H11Handshake:
         """
         return self._connection
 
-    def initiate_upgrade_connection(self, headers: Headers, path: str) -> None:
+    def initiate_upgrade_connection(
+        self, headers: Headers, path: Union[bytes, str]
+    ) -> None:
         """Initiate an upgrade connection.
 
         This should be used if the request has already be received and
@@ -198,15 +200,15 @@ class H11Handshake:
             if name == b"connection":
                 connection_tokens = split_comma_header(value)
             elif name == b"host":
-                host = value.decode("ascii")
+                host = value.decode("idna")
                 continue  # Skip appending to headers
             elif name == b"sec-websocket-extensions":
-                extensions = split_comma_header(value)
+                extensions.extend(split_comma_header(value))
                 continue  # Skip appending to headers
             elif name == b"sec-websocket-key":
                 key = value
             elif name == b"sec-websocket-protocol":
-                subprotocols = split_comma_header(value)
+                subprotocols.extend(split_comma_header(value))
                 continue  # Skip appending to headers
             elif name == b"sec-websocket-version":
                 version = value
@@ -324,7 +326,7 @@ class H11Handshake:
         self._nonce = generate_nonce()
 
         headers = [
-            (b"Host", request.host.encode("ascii")),
+            (b"Host", request.host.encode("idna")),
             (b"Upgrade", b"WebSocket"),
             (b"Connection", b"Upgrade"),
             (b"Sec-WebSocket-Key", self._nonce),
